//=============================================================================
//  MuseScore
//  Linux Music Score Editor
//  $Id:$
//
/******************************************************************************
   Add chosen Musical Cords from existent single notes.

   addchords.js
   Version: 01.02

   Requires: addchords.ui,  addchordshelp.ui

      Opens a window which contains brief details of the current score.
      It can handle a selection or no selection at all:
         If there is a selection then addchords works only on selected notes; 
         If no selection has been made then it works on whole staff 1, that 
         normally is bass staff.

      Note. Only notes in the first voice are processed and there are the 
            root of selected Chord.

   History:
   01.03   Added a lot of Chords into List
   01.02   Add ListWidget to handle selected Chord 
   01.01   Initial Release
   Author: Mauro Conti

   This plugin is based on CreateChords.js of JHE
*********************************************************************************/
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================
//    This is ECMAScript code (ECMA-262 aka "Java Script")
//=============================================================================
var gVersion = '01.04';
var g_title = "Add Chosen Chords From Existing Notes";
var g_formMain;   
var g_formHelp;
var xmc_gCursor;
var g_endTick;
var xmc_gCurrentItem;

// This table contains all information we need to create a Chord relative to the Root Note:
// Names is the mnemonic Chord Name like it is shown in the ChooseChordDialog;
// Grades are how many notes are into Chord.
// Deltax are how many semitones we must add to Root note to get Notex of chosen chord.
// Normally Delta1 is Root Position, so it is 0; only in inverted Chord Delta1 will be 
// different from 0,
// Column index                 0           1            2     3      4      5      6      7      8      9
   var xmc_gChordTable = [//  NameIt      NameEn       Gradi Delta1 Delta2 Delta3 Delta4 Delta5 Delta6 Delta7
                           [ 'none'    , 'none'    ,     0,    0,     0,     0,     0,     0,     0,     0 ],  //  0 
                           [ 'RM'      , 'RM'      ,     3,    0,     4,     7,     0,     0,     0,     0 ],  //  1 
                           [ 'Rm'      , 'Rm4'     ,     3,    0,     3,     7,     0,     0,     0,     0 ],  //  2 
                           [ 'R4'      , 'Rsus4'   ,     3,    0,     5,     7,     0,     0,     0,     0 ],  //  3 
                           [ 'R5-'     , 'Rb5'     ,     3,    0,     4,     6,     0,     0,     0,     0 ],  //  4 
                           [ 'R5+'     , 'Raug'    ,     3,    0,     4,     8,     0,     0,     0,     0 ],  //  5 
                           [ 'Rdim'    , 'Rdim'    ,     3,    0,     3,     6,     0,     0,     0,     0 ],  //  6 
                           [ 'R6'      , 'R6'      ,     4,    0,     4,     7,     9,     0,     0,     0 ],  //  7 
                           [ 'Rm6'     , 'Rm6'     ,     4,    0,     3,     7,     9,     0,     0,     0 ],  //  8 
                           [ 'R7'      , 'R7'      ,     4,    0,     4,     7,    10,     0,     0,     0 ],  //  9 
                           [ 'R7+'     , 'Rmaj7'   ,     4,    0,     4,     7,    11,     0,     0,     0 ],  // 10 
                           [ 'Rm7'     , 'Rm7'     ,     4,    0,     3,     7,    10,     0,     0,     0 ],  // 11 
                           [ 'Rm7+'    , 'Rm(maj7)',     4,    0,     3,     7,    11,     0,     0,     0 ],  // 12 
                           [ 'R7/4'    , 'R7sus4'  ,     4,    0,     5,     7,    10,     0,     0,     0 ],  // 13 
                           [ 'R7dim'   , 'Rdim7'   ,     4,    0,     3,     6,     9,     0,     0,     0 ],  // 14 
                           [ 'R7/5-'   , 'R7b5'    ,     4,    0,     4,     6,    10,     0,     0,     0 ],  // 15 
                           [ 'R7/5+'   , 'R7#5'    ,     4,    0,     4,     8,    10,     0,     0,     0 ],  // 16 
                           [ 'R9'      , 'R9'      ,     5,    0,     4,     7,    10,    14,     0,     0 ],  // 17 
                           [ 'R9-'     , 'R7b9'    ,     5,    0,     4,     7,    10,    13,     0,     0 ],  // 18 
                           [ 'R9+'     , 'R7#9'    ,     5,    0,     4,     7,    10,    15,     0,     0 ],  // 22 
                           [ 'Rm9'     , 'Rm9'     ,     5,    0,     3,     7,    10,    14,     0,     0 ],  // 19 
                           [ 'Rm9-'    , 'Rm7b9'   ,     5,    0,     3,     7,    10,    13,     0,     0 ],  // 20 
                           [ 'Rm9+'    , 'Rm7#9'   ,     5,    0,     3,     7,    10,    15,     0,     0 ],  // 20 
                           [ 'end'     , 'end'     ,     0,    0,     0,     0,     0,     0,     0,     0 ]   // Must Be Last LINE!! DO NOT CHANGE 
                         ];


// ----------------------------------------------------------------------------
function init() 
{
// This function is run at the start up of MuseScore.
};

// ----------------------------------------------------------------------------
// This function is run when the plugin is selected.
function run() 
{
// Local Vars
   var xmc_Index;
   var xmc_CurrentItem;
   
// Verify selection
   xmc_gCursor = new Cursor( curScore );
   
   xmc_gCursor.goToSelectionStart();

   if( xmc_gCursor.eos() == true )
   {
   // Nothing selected so it assumes that you want modify all bass staff
      xmc_gCursor.staff = 1;
      xmcMessage( "You haven't selected any note, so you have selected ALL Staff 1 !" );    
   }

// In each case we handle only on voice 0   
   xmc_gCursor.voice = 0;

// Load main form
   g_formMain = xmcLoadForm(pluginPath, "addchords.ui");

// Details including selection

// Program Version
   g_formMain.labelVersion.setText("V: " + gVersion);
   
// Score Title, Staff and Selected Measures
   if( curScore.title.length == 0 ) 
   {
      xmcQColouredText( g_formMain.textTitle, "376f00", "Untitled", true );
   }
   else 
   {
      xmcQColouredText(g_formMain.textTitle, "376f00", curScore.title, true);
   }
   g_formMain.textStaves.setText("Staves: " + curScore.staves);
   g_formMain.textMeasures.setText("Measures: " + curScore.measures);

   if( xmc_gCursor.staff == 0 ) 
   {
      g_formMain.textSelected.setText( "WARNING! Staff 0 selected" );
   }
   else 
   {
      g_formMain.textSelected.setText( "Staff selected: " + xmc_gCursor.staff );
   }
   
   xmcSelectedMeasureNumbers( xmc_gCursor );
   g_formMain.textMSelected.setText( "Sel. Measures: " + g_selectedMeasures  );

// Connect the Help Button function   
   g_formMain.pushHelp.clicked.connect( helpWanted );                                        

// Connect the Exit Button function
   g_formMain.pushExit.clicked.connect( Exit );                                        

// Connect the SymbolsIt Radio Button function
   g_formMain.symbolsIt.clicked.connect( SymbolsItClicked );                                        

// Connect the SymbolsIt Radio Button function
   g_formMain.symbolsEn.clicked.connect( SymbolsEnClicked );                                        
   
// Connect the ListChordItemClicked function
   g_formMain.listWidget['itemClicked( QListWidgetItem *)'].connect( ListWidgetItemClicked );

   xmc_gCursor.voice = 0;
   
   xmc_gCursor.goToSelectionEnd();
   g_endTick = xmc_gCursor.tick();     // If no selection, this will go to end of score
   xmc_gCursor.goToSelectionStart();

   if( xmc_gCursor.eos() == true ) 
   {
      xmc_gCursor.staff = 1;
      xmc_gCursor.rewind();
   }  // If no selection, start at beginning of score- Staff 1

// Now search next note to use like Chord's Root, and then change to X-Circle its head
   while( xmc_gCursor.tick() < g_endTick ) 
   {
      if( xmc_gCursor.isChord() == true ) 
      {
         if( xmc_gCursor.chord().notes > 1 )
         {   
            xmcMessage( "Warning: You are selecting like CHORD'S ROOT an already existent chord!!!" );    
         }
         xmc_gCursor.chord().note(0).noteHead = 6;
         break;
      }         
      xmc_gCursor.next();
   }
   
   if( xmc_gCursor.tick() != g_endTick )
   {
   // For default it shows Italian Sybols into Chord List
      xmc_Index = 0;
      while( xmc_gChordTable[xmc_Index][0] != 'end' )
      {
         xmc_CurrentItem = g_formMain.listWidget.item( xmc_Index );
         xmc_CurrentItem.setText( xmc_gChordTable[xmc_Index ][0]);
         xmc_Index++;
      }   
   // Show form
      g_formMain.show();
   }
   else
   {
      xmcMessage( "Warning: You haven't selected any note to use like CHORD'S ROOT!" );    
   }
   
   return;
};

// ----------------------------------------------------------------------------
function helpWanted() 
{
// Help pressed on main window or called when nothing selected
   g_formHelp = xmcLoadForm( pluginPath, "addchordshelp.ui" );
   
// Connect the helpContinue function   
   g_formHelp.pushContinue.accepted.connect( helpContinue );

// Program Version
   g_formHelp.Version.setText("V: " + gVersion);

   g_formHelp.show();
   return;
};

// ----------------------------------------------------------------------------
function helpContinue() 
{
// Continue pressed on help form
   g_formHelp.close();
   return;
};

// ----------------------------------------------------------------------------
function Exit() 
{
// Close Main Form and Plugin
   g_formMain.close();
   return;
};

// ----------------------------------------------------------------------------
function SymbolsItClicked() 
{
   // For default it shows Italian Sybols into Chord List
      xmc_Index = 0;
      while( xmc_gChordTable[xmc_Index][0] != 'end' )
      {
         xmc_CurrentItem = g_formMain.listWidget.item( xmc_Index );
         xmc_CurrentItem.setText( xmc_gChordTable[xmc_Index ][0]);
         xmc_Index++;
      }   
   // Show form
//      g_formMain.show();
   return;
};

// ----------------------------------------------------------------------------
function SymbolsEnClicked() 
{
   // For default it shows Italian Sybols into Chord List
      xmc_Index = 0;
      while( xmc_gChordTable[xmc_Index][0] != 'end' )
      {
         xmc_CurrentItem = g_formMain.listWidget.item( xmc_Index );
         xmc_CurrentItem.setText( xmc_gChordTable[xmc_Index ][1]);
         xmc_Index++;
      }   
   // Show form
//      g_formMain.show();
   return;
};


// ----------------------------------------------------------------------------
function ListWidgetItemClicked( ListViewItem )
{
// Local Vars
   var chord;
   var xmc_root;
   var xmc_index;
   var xmc_i;
   var xmc_note;
   var xmc_end;
   var xmc_start;
   var xmc_grades;

   curScore.startUndo();
   while( xmc_gCursor.tick() < g_endTick ) 
   {
      if( xmc_gCursor.isChord() == true ) 
      {
         chord     = xmc_gCursor.chord();
         xmc_root  = chord.note(0);   

      // Grab the index of selected Chord   
         xmc_index = g_formMain.listWidget.currentRow;
         
      // To default no Chord is selected
         g_formMain.listWidget.currentRow = -1;   

      //    
         xmc_start = 4;                                     
      // For now we don't handle inverted chord, so it dislike thee ROOT    
      // This is the reason for " - 1 "  
         xmc_grades = xmc_gChordTable[xmc_index][2] - 1;
         
         xmc_end   = xmc_start + xmc_grades;
         
 
      // It adds all grades into selected Chord    
         for( xmc_i = xmc_start; xmc_i < xmc_end; xmc_i++ )
         {
            xmc_note = new Note( curScore );

            xmc_note.pitch = ( xmc_root.pitch ) + xmc_gChordTable[xmc_index][ xmc_i] ;

            chord.addNote( xmc_note );

            delete( xmc_note );
         }
         
      // Change again to normal the root head
         xmc_gCursor.chord().note(0).noteHead = 0;

         xmc_gCursor.next();
         break;
      }
      xmc_gCursor.next();
   }// while( xmc_gCursor.tick() < g_endTick ) 

// Now search next note to use like Chord's Root, and then change to X-Circle its head
   while( xmc_gCursor.tick() < g_endTick ) 
   {
      if( xmc_gCursor.isChord() == true ) 
      {
         if( xmc_gCursor.chord().notes > 1 )
         {   
            xmcMessage( "Warning: You are selecting like CHORD'S ROOT an already existent chord!!!" );    
         }
         xmc_gCursor.chord().note(0).noteHead = 6;
         break;
      }         
      xmc_gCursor.next();
   }

   curScore.endUndo();
   if( xmc_gCursor.tick() == g_endTick )
   {   
      xmcMessage( "Done: All selected measure was correctly handled!!!" );    
      g_formMain.close();
   }
   return;
};
// ----------------------------------------------------------------------------
// Utility Functions
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
function xmcLoadForm( myPath, myFormName ) 
{
// Returns a form after opening and loading it
// myPath      Path to form e.g. pluginPath
// myFormName  Name and extension of form e.g. "name.iu"
   var loader = new QUiLoader(null);
   var file = new QFile(myPath + "/"+myFormName);
   file.open( QIODevice.OpenMode( QIODevice.ReadOnly, QIODevice.Text ) );
   return( loader.load( file, null) );
};

// ----------------------------------------------------------------------------
function xmcMessage( myText ) 
{
// Output a message
// myText   Main text
   var mb = new QMessageBox();
   mb.setWindowTitle( g_title );
   mb.text = myText;
   mb.exec();
   return;
};

// ----------------------------------------------------------------------------
function xmcQColouredText( myTextObject, myCol, myText, myBold ) 
{
// Add coloured text to form object (QLabel)
// myTextObject   Object e.g. globalformMain.textOne
// myColour       Colour as 6 didgit hexadecimal text string
//                e.g. "00007f" Dark blue, "0000ff" Blue, "376f00" Dark Green
//                "008080" Turquoise, "800040" Dark Purple, "ff0040" Dark Red,
//                "ff8000" Orange, "ff0000" Red, "000000" Black
// myText         Text
// myBold         true if colour to be bold
   var string1a = "<span style= \"color:#";
   var string1b = "<span style= \"font-weight:600; color:#";
   var string2 = "\">";
   var string3 = "</span>";

   if( myBold )
   {
      myTextObject.setText( string1b + myCol + string2 + myText + string3 );
      
   }
   else
   {
      myTextObject.setText( string1a + myCol + string2 + myText + string3 );
      
   }
   return;
};

// ----------------------------------------------------------------------------
function xmcSelectedMeasureNumbers( myCursor ) 
{
// Puts the number of the first and last selected measures into global variables
//	myCursor	Cursor
//	Needs global variables: var g_firstMeasure; var g_lastMeasure; g_selectedMeasures (optional)
// Warning: the cursor is left at the measure after the last selected one
   var firstTick;       // First selected measure tick position
   var lastTick;        // Last selected measure tick position
   var firstMeasure;    // Number of first measure selected
   var lastMeasure;     // Number of last measure selected
   var measureNumber;   // Measure number from 1

   firstMeasure = 1;
   lastMeasure  = curScore.measures;

// lastTick conterrà comunque l'ultimo tick, della selezione o dello score
   myCursor.goToSelectionEnd();
   lastTick = myCursor.tick();

// firstTick conterrà comunque i primo tick, della selezione o dello score
// e comunque il cursore sarà posizionato alla posizione del tick iniziale. 
   myCursor.goToSelectionStart();
   if( myCursor.eos() == true ) 
   {
   // If no selection then full score   
      myCursor.rewind();
   }
   firstTick = myCursor.tick();

// Get first measure number
   measureNumber = 1;

// Ora mi posiziono all'inizio dello score   
   myCursor.rewind();

// Conto tutte le measure prima della selezione
   while( myCursor.tick() < firstTick ) 
   {
      measureNumber = measureNumber + 1;
      myCursor.nextMeasure();
   }

// firstMeasure contiene il numero della prima measure selezionata
   firstMeasure = measureNumber;

// Get last selected measure number
   while( myCursor.tick() < lastTick ) 
   {
      measureNumber = measureNumber + 1;
      myCursor.nextMeasure();
   }
   lastMeasure = measureNumber;

   g_selectedMeasures = lastMeasure - firstMeasure;
   return;
};

// ----------------------------------------------------------------------------
var mscorePlugin = 
{
// Defines were the function will be placed in the MuseScore menu structure
      menu: 'Plugins.Add Selected Chords',
      init: init,
      run:  run
};

mscorePlugin;